@@ -30,21 +30,27 @@ namespace GtkSharp.Generation {
 
 	public class Signal {
 
-		private string name;
-		private XmlElement elem;
-		private ReturnValue retval;
-		private Parameters parms;
-		private ClassBase container_type;
+		bool marshaled;
+		string name;
+		XmlElement elem;
+		ReturnValue retval;
+		Parameters parms;
+		ClassBase container_type;
 
 		public Signal (XmlElement elem, ClassBase container_type)
 		{
 			this.elem = elem;
 			name = elem.GetAttribute ("name");
+			marshaled = elem.GetAttribute ("manual") == "true";
 			retval = new ReturnValue (elem ["return-type"]);
 			parms = new Parameters (elem["parameters"]);
 			this.container_type = container_type;
 		}
 
+		bool Marshaled {
+			get { return marshaled; }
+		}
+
 		public string Name {
 			get {
 				return name; 
@@ -85,13 +91,7 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		public string CallbackName {
-			get {
-				return Name + "SignalCallback";
-			}
-		}
-
-		private string CallbackSig {
+		string CallbackSig {
 			get {
 				string result = "";
 				for (int i = 0; i < parms.Count; i++) {
@@ -107,11 +107,13 @@ namespace GtkSharp.Generation {
 				return result;
 			}
 		}
-				
-		public string DelegateName {
-			get {
-				return Name + "SignalDelegate";
-			}
+
+		string CallbackName {
+			get { return Name + "SignalCallback"; }
+		}
+
+		string DelegateName {
+			get { return Name + "SignalDelegate"; }
 		}
 
                 private string EventArgsName {
@@ -152,9 +154,9 @@ namespace GtkSharp.Generation {
                         }
                 }
 
-		public string ClassFieldName {
+		string ClassFieldName {
 			get {
-				return elem.GetAttribute("cname").Replace ("-", "_");
+				return elem.HasAttribute ("field_name") ? elem.GetAttribute("field_name") : String.Empty;
 			}
 		}
 
@@ -168,7 +170,7 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		public bool IsEventHandler {
+		private bool IsEventHandler {
 			get {
 				return retval.CSType == "void" && parms.Count == 1 && (parms [0].Generatable is ObjectGen || parms [0].Generatable is InterfaceGen);
 			}
@@ -308,12 +310,18 @@ namespace GtkSharp.Generation {
 				sw.WriteLine ("\t\tpublic " + parms[i].CSType + " " + parms[i].StudlyName + "{");
 				if (parms[i].PassAs != "out") {
 					sw.WriteLine ("\t\t\tget {");
-					sw.WriteLine ("\t\t\t\treturn (" + parms[i].CSType + ") Args[" + (i - 1) + "];");
+					if (SymbolTable.Table.IsInterface (parms [i].CType))
+						sw.WriteLine ("\t\t\t\treturn {0}Adapter.GetObject (Args [{1}] as GLib.Object);", parms [i].CSType, i - 1);
+					else
+						sw.WriteLine ("\t\t\t\treturn ({0}) Args [{1}];", parms [i].CSType, i - 1);
 					sw.WriteLine ("\t\t\t}");
 				}
 				if (parms[i].PassAs != "") {
 					sw.WriteLine ("\t\t\tset {");
-					sw.WriteLine ("\t\t\t\tArgs[" + (i - 1) + "] = (" + parms[i].CSType + ")value;");
+					if (SymbolTable.Table.IsInterface (parms [i].CType))
+						sw.WriteLine ("\t\t\t\tArgs [{0}] = value is {1}Adapter ? (value as {1}Adapter).Implementor : value;", i - 1, parms [i].CSType);
+					else
+						sw.WriteLine ("\t\t\t\tArgs[" + (i - 1) + "] = (" + parms[i].CSType + ")value;");
 					sw.WriteLine ("\t\t\t}");
 				}
 				sw.WriteLine ("\t\t}");
@@ -324,7 +332,7 @@ namespace GtkSharp.Generation {
 			sw.Close ();
 		}
 
-		private void GenVirtualMethod (StreamWriter sw, ClassBase implementor)
+		private void GenVMDeclaration (StreamWriter sw, ClassBase implementor)
 		{
 			VMSignature vmsig = new VMSignature (parms);
 			sw.WriteLine ("\t\t[GLib.DefaultSignalHandler(Type=typeof(" + (implementor != null ? implementor.QualifiedName : container_type.QualifiedName) + "), ConnectionMethod=\"Override" + Name +"\")]");
@@ -332,6 +340,101 @@ namespace GtkSharp.Generation {
 			if (NeedNew (implementor))
 				sw.Write ("new ");
 			sw.WriteLine ("virtual {0} {1} ({2})", retval.CSType, "On" + Name, vmsig.ToString ());
+		}
+
+		private string CastFromInt (string type)
+		{
+			return type != "int" ? "(" + type + ") " : "";
+		}
+
+		private string GlueCallString {
+			get {
+				string result = "Handle";
+
+				for (int i = 1; i < parms.Count; i++) {
+					Parameter p = parms [i];
+					IGeneratable igen = p.Generatable;
+
+					if (i > 1 && parms [i - 1].IsString && p.IsLength && p.PassAs == String.Empty) {
+						string string_name = parms [i - 1].Name;
+						result += ", " + igen.CallByName (CastFromInt (p.CSType) + "System.Text.Encoding.UTF8.GetByteCount (" +  string_name + ")");
+						continue;
+					}
+
+					p.CallName = p.Name;
+					string call_parm = p.CallString;
+
+					if (p.IsUserData && parms.IsHidden (p) && !parms.HideData && (i == 1 || parms [i - 1].Scope != "notified")) {
+						call_parm = "IntPtr.Zero"; 
+					}
+
+					result += ", " + call_parm;
+				}
+				return result;
+			}
+		}
+
+		private string GlueSignature {
+			get {
+				string result = String.Empty;
+				for (int i = 0; i < parms.Count; i++)
+					result += parms[i].CType.Replace ("const-", "const ") + " " + parms[i].Name + (i == parms.Count-1 ? "" : ", ");
+				return result;
+			}
+		}
+
+		private string DefaultGlueValue {
+			get {
+				string val = retval.DefaultValue;
+				switch (val) {
+				case "null":
+					return "NULL";
+				case "false":
+					return "FALSE";
+				case "true":
+					return "TRUE";
+				default:
+					return val;
+				}
+			}
+		}
+
+		private void GenGlueVirtualMethod (GenerationInfo gen_info)
+		{
+			StreamWriter glue = gen_info.GlueWriter;
+			string glue_name = String.Format ("{0}sharp_{1}_base_{2}", container_type.NS.ToLower ().Replace (".", "_"), container_type.Name.ToLower (), ClassFieldName);
+			glue.WriteLine ("{0} {1} ({2});\n", retval.CType, glue_name, GlueSignature);
+			glue.WriteLine ("{0}\n{1} ({2})", retval.CType, glue_name, GlueSignature);
+			glue.WriteLine ("{");
+			glue.WriteLine ("\t{0}Class *klass = ({0}Class *) get_threshold_class (G_OBJECT ({1}));", container_type.CName, parms[0].Name);
+			glue.Write ("\tif (klass->{0})\n\t\t", ClassFieldName);
+			if (!IsVoid)
+				glue.Write ("return ");
+			glue.Write ("(* klass->{0}) (", ClassFieldName);
+			for (int i = 0; i < parms.Count; i++)
+				glue.Write (parms[i].Name + (i == parms.Count - 1 ? "" : ", "));
+			glue.WriteLine (");");
+			if (!IsVoid)
+				glue.WriteLine ("\treturn " + DefaultGlueValue + ";");
+			glue.WriteLine ("}");
+
+			StreamWriter sw = gen_info.Writer;
+			sw.WriteLine ("\t\t[DllImport (\"{0}\")]", gen_info.GluelibName);
+			sw.WriteLine ("\t\tstatic extern {0} {1} ({2});\n", retval.MarshalType, glue_name, parms.ImportSignature);
+			GenVMDeclaration (sw, null);
+			sw.WriteLine ("\t\t{");
+			MethodBody body = new MethodBody (parms);
+			body.Initialize (gen_info, false, false, String.Empty);
+			sw.WriteLine ("\t\t\t{0}{1} ({2});", IsVoid ? "" : retval.MarshalType + " __ret = ", glue_name, GlueCallString);
+			body.Finish (sw, "");
+			if (!IsVoid)
+				sw.WriteLine ("\t\t\treturn {0};", retval.FromNative ("__ret"));
+			sw.WriteLine ("\t\t}\n");
+		}
+
+		private void GenChainVirtualMethod (StreamWriter sw, ClassBase implementor)
+		{
+			GenVMDeclaration (sw, implementor);
 			sw.WriteLine ("\t\t{");
 			if (IsVoid)
 				sw.WriteLine ("\t\t\tGLib.Value ret = GLib.Value.Empty;");
@@ -384,14 +487,37 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\t}\n");
 		}
 
-		private void GenDefaultHandlerDelegate (StreamWriter sw, ClassBase implementor)
+		private void GenDefaultHandlerDelegate (GenerationInfo gen_info, ClassBase implementor)
 		{
-			ManagedCallString call = new ManagedCallString (parms);
+			StreamWriter sw = gen_info.Writer;
+			StreamWriter glue;
+			bool use_glue = gen_info.GlueEnabled && implementor == null && ClassFieldName.Length > 0;
+			string glue_name = String.Empty;
+			ManagedCallString call = new ManagedCallString (parms, true);
 			sw.WriteLine ("\t\t[GLib.CDeclCallback]");
 			sw.WriteLine ("\t\tdelegate " + retval.ToNativeType + " " + Name + "VMDelegate (" + parms.ImportSignature + ");\n");
+
+			if (use_glue) {
+				glue = gen_info.GlueWriter;
+				glue_name = String.Format ("{0}sharp_{1}_override_{2}", container_type.NS.ToLower ().Replace (".", "_"), container_type.Name.ToLower (), ClassFieldName);
+				sw.WriteLine ("\t\t[DllImport (\"{0}\")]", gen_info.GluelibName);
+				sw.WriteLine ("\t\tstatic extern void {0} (IntPtr gtype, {1}VMDelegate cb);\n", glue_name, Name);
+				glue.WriteLine ("void {0} (GType gtype, gpointer cb);\n", glue_name);
+				glue.WriteLine ("void\n{0} (GType gtype, gpointer cb)", glue_name);
+				glue.WriteLine ("{");
+				glue.WriteLine ("\tGObjectClass *klass = g_type_class_peek (gtype);");
+				glue.WriteLine ("\tif (klass == NULL)");
+				glue.WriteLine ("\t\tklass = g_type_class_ref (gtype);");
+				glue.WriteLine ("\t(({0} *)klass)->{1} = cb;", container_type.CName + "Class", ClassFieldName);
+				glue.WriteLine ("}\n");
+			}
+
 			sw.WriteLine ("\t\tstatic {0} {1};\n", Name + "VMDelegate", Name + "VMCallback");
 			sw.WriteLine ("\t\tstatic " + retval.ToNativeType + " " + Name.ToLower() + "_cb (" + parms.ImportSignature + ")");
 			sw.WriteLine ("\t\t{");
+			string unconditional = call.Unconditional ("\t\t\t");
+			if (unconditional.Length > 0)
+				sw.WriteLine (unconditional);
 			sw.WriteLine ("\t\t\ttry {");
 			sw.WriteLine ("\t\t\t\t{0} {1}_managed = GLib.Object.GetObject ({1}, false) as {0};", implementor != null ? implementor.Name : container_type.Name, parms[0].Name);
 			sw.Write (call.Setup ("\t\t\t\t"));
@@ -413,13 +539,21 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\t{");
 			sw.WriteLine ("\t\t\tif (" + Name + "VMCallback == null)");
 			sw.WriteLine ("\t\t\t\t" + Name + "VMCallback = new " + Name + "VMDelegate (" + Name.ToLower() + "_cb);");
-			sw.WriteLine ("\t\t\tOverrideVirtualMethod (gtype, " + CName + ", " + Name + "VMCallback);");
+			if (use_glue)
+				sw.WriteLine ("\t\t\t{0} (gtype.Val, {1}VMCallback);", glue_name, Name);
+			else
+				sw.WriteLine ("\t\t\tOverrideVirtualMethod (gtype, " + CName + ", " + Name + "VMCallback);");
 			sw.WriteLine ("\t\t}\n");
 		}
 
 		public void GenEvent (StreamWriter sw, ClassBase implementor, string target)
 		{
-			string marsh = IsEventHandler ? "" : ", new " + DelegateName + "(" + CallbackName + ")";
+			string args_type = IsEventHandler ? "" : ", typeof (" + EventArgsQualifiedName + ")";
+			
+			if (Marshaled) {
+				GenCallback (sw);
+				args_type = ", new " + DelegateName + "(" + CallbackName + ")";
+			}
 
 			sw.WriteLine("\t\t[GLib.Signal("+ CName + ")]");
 			sw.Write("\t\tpublic ");
@@ -427,11 +561,11 @@ namespace GtkSharp.Generation {
 				sw.Write("new ");
 			sw.WriteLine("event " + EventHandlerQualifiedName + " " + Name + " {");
 			sw.WriteLine("\t\t\tadd {");
-			sw.WriteLine("\t\t\t\tGLib.Signal sig = GLib.Signal.Lookup (" + target + ", " + CName + marsh + ");");
+			sw.WriteLine("\t\t\t\tGLib.Signal sig = GLib.Signal.Lookup (" + target + ", " + CName + args_type + ");");
 			sw.WriteLine("\t\t\t\tsig.AddDelegate (value);");
 			sw.WriteLine("\t\t\t}");
 			sw.WriteLine("\t\t\tremove {");
-			sw.WriteLine("\t\t\t\tGLib.Signal sig = GLib.Signal.Lookup (" + target + ", " + CName + marsh + ");");
+			sw.WriteLine("\t\t\t\tGLib.Signal sig = GLib.Signal.Lookup (" + target + ", " + CName + args_type + ");");
 			sw.WriteLine("\t\t\t\tsig.RemoveDelegate (value);");
 			sw.WriteLine("\t\t\t}");
 			sw.WriteLine("\t\t}");
@@ -445,9 +579,11 @@ namespace GtkSharp.Generation {
 			if (implementor == null)
 				GenEventHandler (gen_info);
 
-			GenCallback (sw);
-			GenDefaultHandlerDelegate (sw, implementor);
-			GenVirtualMethod (sw, implementor);
+			GenDefaultHandlerDelegate (gen_info, implementor);
+			if (gen_info.GlueEnabled && implementor == null && ClassFieldName.Length > 0)
+				GenGlueVirtualMethod (gen_info);
+			else
+				GenChainVirtualMethod (sw, implementor);
 			GenEvent (sw, implementor, "this");
 			
 			Statistics.SignalCount++;
