@@ -33,11 +33,11 @@ namespace GLib {
 			this.errptr = errptr;
 		}
 
-		[DllImport("glibsharpglue")]
+		[DllImport("glibsharpglue-2")]
 		static extern IntPtr gtksharp_error_get_message (IntPtr errptr);
 		public override string Message {
 			get {
-				return Marshal.PtrToStringAnsi (gtksharp_error_get_message (errptr));
+				return Marshaller.Utf8PtrToString (gtksharp_error_get_message (errptr));
 			}
 		}
 
