@@ -38,6 +38,30 @@ namespace GLib {
 				proxy_handler = new TimeoutHandlerInternal (Handler);
 			}
 
+			~TimeoutProxy ()
+			{
+				Dispose (false);
+			}
+
+			public void Dispose ()
+			{
+				Dispose (true);
+				GC.SuppressFinalize (this);
+			}
+
+			protected virtual void Dispose (bool disposing)
+			{
+				// Both branches remove our delegate from the
+				// managed list of handlers, but only
+				// Source.Remove will remove it from the
+				// unmanaged list also.
+
+				if (disposing)
+					Remove ();
+				else
+					Source.Remove (ID);
+			}
+
 			public bool Handler ()
 			{
 				try {
@@ -62,11 +86,11 @@ namespace GLib {
 		{
 			TimeoutProxy p = new TimeoutProxy (hndlr);
 
-			uint code = g_timeout_add (interval, (TimeoutHandlerInternal) p.proxy_handler, IntPtr.Zero);
+			p.ID = g_timeout_add (interval, (TimeoutHandlerInternal) p.proxy_handler, IntPtr.Zero);
 			lock (Source.source_handlers)
-				Source.source_handlers [code] = p;
+				Source.source_handlers [p.ID] = p;
 
-			return code;
+			return p.ID;
 		}
 	}
 }
